#ifndef __SNAP_DEFINES__
#include "snap_defines.h"
#endif

#ifndef __SNAP_BASE__
#include "snap_base.h"
#endif

class crc16;
class checksum8;

class snap_rx: public snap_base {

 public:
  snap_rx::snap_rx( int my_address, int debug );
  snap_rx::~snap_rx();

  // Reset everything. Use after rx of each packet
  void snap_rx::init_rx( void );

  // Send a byte at a time into the packet processor, from
  // "The Wire"
  void snap_rx::byte_in( unsigned char byte_in );

  // Find out the starting address of the buffer that has the
  // finished, processed packet in it. This is the start of
  // the data that is pointed to
  unsigned char * snap_rx::packet_out( void );

  // Length of the data payload
  int snap_rx::packet_out_len( void );

  // Is there a packet in the buffer that is ready?
  unsigned char snap_rx::packet_ready_query( void );

  // get the dab
  int snap_rx::get_dab_data( void );

  // get the sab
  int snap_rx::get_sab_data( void );

  // get the pfb
  int snap_rx::get_pfb_data( void );

  // Dump packets that are not for my address? Default = YES
  void snap_rx::set_auto_dump( int auto_dump_in );

  // Show rx class packet stats
  void snap_rx::show_stats( void );

  // display the data in raw packet form for debugging
  void snap_rx::display_raw_packet( void );

 private:
  int my_packet;
  int broadcast;
  unsigned char snap_rx::sync_detected;
  int snap_rx::ack_state;
  int snap_rx::cmd_mode;
  int snap_rx::err_len;
  int snap_rx::data_len;
  int snap_rx::auto_dump;
  unsigned char snap_rx::rx_buffer[ RX_BUFFER_SIZE ];
  unsigned char snap_rx::packet_ready;
  int snap_rx::total_header_length;
  int snap_rx::error_length;
  int snap_rx::rx_buffer_offset;
  int snap_rx::byte_in_count;
  int snap_rx::total_packet_length;
  int snap_rx::starting_offset;
  int snap_rx::reported_buffer_len;
  unsigned int snap_rx::calc_header;
  void snap_rx::calc_mode_and_size( void );
  int snap_rx::calc_ndb( unsigned char packet_ndb );
  void snap_rx::confirm_error( void );
  void snap_rx::do_error( unsigned char byte_in );
  void snap_rx::extract_header_data( void );

  // Stats vars
  int snap_rx::stat_bytes;
  int snap_rx::stat_good_packets;
  int snap_rx::stat_bad_packets;
  int snap_rx::stat_payload_bytes;
  int snap_rx::stat_dropped_packets;
  int snap_rx::stat_broadcast_packets;
};
