// snap_rx / snap_tx class test bench file
// Carl D. Walker
// Release under GPL
// 03-20-99
//
// Updated a bit to use threads
//
#include "snap_tx.h"
#include "snap_rx.h"
#include "snap_defines.h"
//
#include <pthread.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <iostream.h>
#include <time.h>

// These must be globals so that the threads can see them
snap_tx         *tx;
snap_rx         *rx;
time_t          local_time;
unsigned char   *rx_packet;
unsigned char   byte;
char            data_2_send[1024];
char            *tx_packet;
int             rx_packet_length;
int             n;
int             i;
int             tx_packet_length;
int             read_len;
int             node_address;
int             rx_active;

//This is the thread that looks at the packet ready information
//and deals with that accordingly
void watch_rx_done( int *loop_count ) {
  int query = 0;
  
  cout << "Receiver ready thread ACTIVE" << endl;
  rx_active = 1; // Here we go...

  while ( 1 ) { // keep the thread alive forever
    // check the state of the ready bit
    query = rx->packet_ready_query();
    if ( query ) {
      // There is a packet ready
      rx_packet = rx->packet_out(); // get the pointer to the packet payload
      rx_packet_length = rx->packet_out_len(); // and the length
      
      //use the raw packet debugging stream
      rx->display_raw_packet();
      // add a new line
      cout << endl;
      // print out the line that we just got
      for ( n = 0; n < rx_packet_length; n++ ) {
	byte = *rx_packet;
	rx_packet++;
	cout << (unsigned char)byte;
      }
      // display the packet stats
      rx->show_stats();
      query = 0; // reset flag
    }
    rx->init_rx(); //reset the rx so that it is ready for the next packet.
    usleep(10000);
    (*loop_count)++; // bump the loop counter so that the peer
    // threads can see that we are still alive and kicking
  } // end of while (...)
}

// This is the thread that sends packets to the receiver class
void transmit( int *loop_count ) {

  // We should be nice and wait for the receiver thread to start
  // before we start sending packets.
  while ( rx_active == 0 ) {
    cout << "Waiting for the rx thread to become active." << endl;
    usleep( 10000 );
  }

  cout << "Transmitter thread ACTIVE" << endl;
  while ( 1 ) {
    tx->set_dab_data( node_address );
    // get the time
    time( &local_time );
    // convert it to ASCII
    strncpy( (char *)&data_2_send, (char *)ctime( &local_time), 
	     strlen( (char *)ctime( &local_time ) ) );
    
    //truncate junk at the end of the string
    *(data_2_send + 25) = 0x0;
    
    read_len = strlen( (char *)&data_2_send );
    tx->tx_packet_done(); // clear the transmit buffers
    tx->tx_data( (char *)&data_2_send, read_len ); // a 'read_len' byte packet
    tx_packet = tx->comp_packet();
    tx_packet_length = tx->get_packet_len();
    // Now we have a packet in the transmit buffer, ready to copy, byte by byte
    // to the receive buffer. Oh, boy.
    for ( i = 0; i < tx_packet_length; i++ ) {
      byte = *tx_packet;
      tx_packet++;
      // put each byte in the rx buffer
      rx->byte_in( byte );
    } // all the bytes of the packet should be in there now!
    
    sleep( 1 );
    
    // Wiggle the node address so that the receiver will
    // be forced to drop packets, as they will
    // be to the 'wrong' address sometimes.
    node_address++;
    if ( node_address > 2 ) {
      node_address = 0;
    }
    (*loop_count)++; // bump the thread loop counter
  }
}

// Program proper follows.....
int main () {

  // thread stuff
  int rx_check_thread_cnt;
  pthread_t rx_check_thread;
  int transmitter_thread_cnt;
  pthread_t transmitter_thread;

  // clear the transmit side (test bench) string buffer
  memset( &data_2_send, 0, 1024 );
  
  // add the snap encoder / decoder classes 
  tx = new snap_tx( 1, 0 );
  rx = new snap_rx( 2, 0 );
 
  rx_check_thread_cnt = 0; // zero the counts
  transmitter_thread_cnt = 0;
  rx_active = 0; // Receiver thread active flag, set to invalid

  tx->set_node_address( 1 );
  tx->set_sab_len( ADDR_8_BIT );
  tx->set_dab_len( ADDR_8_BIT );
  tx->set_edm( WORD_CRC_ERR_MODE );
  
  
  // Fire off the receiver ready checking thread
  pthread_create( &rx_check_thread, 
		  NULL, 
		  (void *)watch_rx_done, 
		  (int *)&rx_check_thread_cnt);

  // Fire off the transmitter thread
  pthread_create( &transmitter_thread, 
		  NULL, 
		  (void *)transmit, 
		  (int *)&transmitter_thread_cnt);

  int minutes = 0;
  while ( 1 ) {
    // loop here forever and let the two threads do all the work
    sleep(60);
    minutes++;
    cout << endl << "Top level alive for " 
	 << minutes << " minutes" << endl;
    cout << "RX Thread loop count:   " << rx_check_thread_cnt
	 << endl;
    cout << "TX Thread loop count:   " << transmitter_thread_cnt
	 << endl << endl;
  }
}

